#ifndef DATA_HPP
#define DATA_HPP

#include <list>
#include <cstdio>

/// data.hpp
/// Definitions of data structures used for holding the result of parsing a program


/// A term in an expression: either an argument like 'n' or a number like 123
/// May have a negative sign (if it was supplied as -n or -123)
struct term
{
	bool neg;
	enum { number_type, arg_type } type;
	union {
		int number;
		char arg;
	};
};
void print(std::FILE*, term const&);

/// An expression: A list of terms.
typedef std::list<term> exp;
void print(std::FILE*, exp const&);

struct param;

/// A statement: an 's' 'r' 'l', a funcall or an argument used as a statement.
struct stem
{
	enum { s, r, l, fun_call_type, arg_type } type;

	/// The name of the function or arg
	char name;

	/// Eventual params passed to the called function
	std::list<param> params;
};
typedef std::list<stem> stats;
void print(std::FILE*, stats const&);
void print(std::FILE*, stem const&);

/// A parameter of a function call: either an expression (1+3-n) or a list
/// of statements (ssslsf(3,4))
struct param
{
	enum  { exp_type, stat_type } type;
	std::list<term> exp;
	std::list<stem> stats;
};
typedef std::list<param> params;

void print(std::FILE*, param const&);
void print(std::FILE*, params const&);

/// A function defition: a name, and argument list and a body
struct function
{
	char name;
	std::list<char> argument;
	std::list<stem> stats;
};
void print(std::FILE*, function const&);

#endif
