#include <algorithm>

#include "level.hpp"

level::level()
{ reset(); }

void level::reset()
{
    std::fill(&cells[0][0],
        &cells[conf::playground_unit_count_x][conf::playground_unit_count_y],
        cell::dot);
    robot_x=conf::playground_unit_count_x/2;
    robot_y=conf::playground_unit_count_y/2;
    robot_dir=direction::up;
}

dyn_level::dyn_level(direction::type dir, int rx, int ry)
{ reset(dir, rx, ry); }

void dyn_level::reset(direction::type dir, int rx, int ry)
{
    robot_dir=dir;
    robot_x=rx;
    robot_y=ry;

    std::fill(&cells[0][0],
        &cells[conf::playground_unit_count_x][conf::playground_unit_count_y],
        0);
}

bool read(level& lev, std::FILE* fp)
{
    char c;
    int x=0, y=0;

    lev.reset();
    while(fscanf(fp, " %c ", &c)==1){
        switch(c){
            case 'o': lev.cells[x][y]=level::cell::white; break;
            case '*': lev.cells[x][y]=level::cell::gray; break;
            case 'x': lev.cells[x][y]=level::cell::wall; break;
            case 'u': lev.robot_x=x; lev.robot_y=y; lev.robot_dir=direction::up; break;
            case 'd': lev.robot_x=x; lev.robot_y=y; lev.robot_dir=direction::down; break;
            case 'l': lev.robot_x=x; lev.robot_y=y; lev.robot_dir=direction::left; break;
            case 'r': lev.robot_x=x; lev.robot_y=y; lev.robot_dir=direction::right; break;
            default: break;
        }
        x++;
        if(x==conf::playground_unit_count_x){
            y++;
            x=0;
            if(y==conf::playground_unit_count_y) return true;
        }
    }
    return false;
}

void dyn_level::unpress_all()
{
    for(int x=0; x<conf::playground_unit_count_x; ++x)
    for(int y=0; y<conf::playground_unit_count_y; ++y)
        cells[x][y] &= ~(cell_info::pressed);
}
